# Simply import the VTK code instead of maintaing another copy.
# If there are additions that are needed here, they should be sent upstream
# into VTK.
from vtk.wx.wxVTKRenderWindowInteractor import *
