{ 'array':

array([('MMMMDCLXXII',), ('MMDCCCLXXXIII',), ('MMMDLXVIIII',),
       ('MMMMDXCV',), ('DCCCLXXII',), ('MMCCCVI',), ('MMMCDLXXXVII',),
       ('MMMMCCXXI',), ('MMMCCXX',), ('MMMMDCCCLXXIII',), ('MMMCCXXXVII',),
       ('MMCCCLXXXXIX',), ('MDCCCXXIIII',), ('MMCXCVI',), ('CCXCVIII',),
       ('MMMCCCXXXII',), ('MDCCXXX',), ('MMMDCCCL',), ('MMMMCCLXXXVI',),
       ('MMDCCCXCVI',), ('MMMDCII',), ('MMMCCXII',), ('MMMMDCCCCI',),
       ('MMDCCCXCII',), ('MDCXX',)],
      dtype=[('Column 1', 'S14')]),

  'kwds': { 'comments': '#',
            'delimiter': None,
            'dtype': { 'formats': ('S14',),
                       'names': ('Column 1',)},
            'skiprows': 0
            }
  }
